import React, { useState } from "react";
import axios from "axios";
import swal from "sweetalert";
import { isAutheticated } from "src/auth";
import { useNavigate } from "react-router-dom";
import { toast } from "react-hot-toast";
import { Button } from "@mui/material";
const AddMultiplerd = () => {
  const [file, setFile] = useState(null);
  const [loading, setLoading] = useState(false);
  const [errors, setErrors] = useState([]);
  const [newlyCreated, setNewlyCreated] = useState([]);
  const [updatedDistributors, setUpdatedDistributors] = useState([]);
  const navigate = useNavigate();
  const token = isAutheticated();
  const handleFileChange = (e) => {
    const selectedFile = e.target.files[0];
    if (
      selectedFile &&
      selectedFile.type ===
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    ) {
      setFile(selectedFile);
    } else {
      swal("Error", "Please upload a valid Excel file", "error");
      setFile(null);
    }
  };

  const handleSubmit = async (e) => {
    e.preventDefault();
    if (!file) {
      toast.error("Please select a file to upload");
      return;
    }
    // console.log(file);
    // console.log(token);
    setLoading(true);
    setErrors([]);
    setNewlyCreated([]);
    setUpdatedDistributors([]);
    try {
      const formData = new FormData();
      formData.append("file", file);

      const { data } = await axios.post(
        "/api/retaildistributor/upload",
        formData,
        {
          headers: {
            "Content-Type": "multipart/form-data",
            Authorization: `Bearer ${token}`,
          },
        }
      );
      // console.log(data);
      if (data.errors && data.errors.length > 0) {
        setErrors(data.errors);
      }
      if (data.newlyCreated && data.newlyCreated.length > 0) {
        setNewlyCreated(data.newlyCreated);
        // console.log(data.newlyCreated);
      }
      if (data.updatedDistributors && data.updatedDistributors.length > 0) {
        setUpdatedDistributors(data.updatedDistributors);
        // console.log(data.updatedDistributors);
      }

      // Redirect or display success message
      if (data.errors && data.errors.length > 0) {
        setErrors(data.errors);
        swal({
          title: "SpreadSheet Upload Successful",
          text: "A few Retailers have errors. Please fix them and upload again.",
          icon: "warning",
          button: "OK",
        });
      } else if (
        data.newlyCreated.length > 0 ||
        data.updatedDistributors.length > 0
      ) {
        swal({
          title: "SpreadSheet Upload Successful",
          text: "Retailers added successfully.",
          icon: "success",
          buttons: "OK",
        });
      } else {
        toast.success("File processed successfully with no new entries.");
        navigate("/retail-distributor");
      }
      setFile(null); // Clear the file input
      document.querySelector('input[type="file"]').value = "";
    } catch (error) {
      console.error("Upload error:", error);
      swal(
        "Error",
        `Failed to upload Retailers: ${
          error.response?.data?.message || "An unexpected error occurred"
        }`,
        "error"
      );
    } finally {
      setLoading(false);
    }
  };

  return (
    <div className="container mt-4">
      <div className="mb-6">
        <Button
          onClick={() => navigate("/retail-distributor")}
          variant="contained"
          color="secondary"
          style={{
            fontWeight: "bold",
            textTransform: "capitalize",
          }}
        >
          Back
        </Button>
      </div>
      <h5 className="mb-6 mt-4">Add Multiple Retailers</h5>
      <div className="my-3">
        <div className="row">
          <div className="col-lg-9">
            <input
              type="file"
              name="file"
              className="form-control"
              accept=".xlsx"
              onChange={handleFileChange}
            />
          </div>
          <div className="col-lg-3">
            <button
              className="btn btn-primary"
              onClick={handleSubmit}
              disabled={loading}
            >
              {loading ? "Uploading..." : "Upload"}
            </button>
          </div>
        </div>
        <p className="pt-1 pl-2 text-secondary">Upload only .xlsx files*</p>
      </div>

      {errors.length > 0 && (
        <div className="my-4">
          <h6>Finding errors while adding the Retailers.</h6>
          <table className="table table-bordered">
            <thead>
              <tr>
                <th>Retailer Name</th>
                <th>Email</th>
                <th>Phone</th>
                <th>PAN</th>
                <th>GST</th>
                <th>AAdhar</th>
                <th>Message</th>
              </tr>
            </thead>
            <tbody>
              {errors.map((error, index) => (
                <tr key={index}>
                  <td>{error.name || "N/A"}</td>
                  <td>{error.email || "N/A"}</td>
                  <td>{error.phone || "N/A"}</td>
                  <td>{error.panNumber || "N/A"}</td>
                  <td>{error.gstNumber || "N/A"}</td>
                  <td>{error.AadharNumber || "N/A"}</td>
                  <td>{error.message}</td>
                </tr>
              ))}
            </tbody>
          </table>
        </div>
      )}
      {updatedDistributors.length > 0 && (
        <div className="my-4">
          <h6>Updated Retailers</h6>
          <table className="table table-bordered">
            <thead>
              <tr>
                <th>Retailer Name</th>
                <th>Email</th>
                <th>Phone</th>
                <th>PAN</th>
                <th>GST</th>
                <th>Aadhar</th>
                <th>Message</th>
              </tr>
            </thead>
            <tbody>
              {updatedDistributors.map((distributor, index) => (
                <tr key={index}>
                  <td>{distributor.name || "N/A"}</td>
                  <td>{distributor.email || "N/A"}</td>
                  <td>{distributor.mobile_number || "N/A"}</td>
                  <td>{distributor.pan_number || "N/A"}</td>
                  <td>{distributor.gst_number || "N/A"}</td>
                  <td>{distributor.aadhar_number || "N/A"}</td>
                  <td>{distributor.updatedFields}</td>
                </tr>
              ))}
            </tbody>
          </table>
        </div>
      )}
      {newlyCreated.length > 0 && (
        <div className="my-4">
          <h6>Newly Created Retailers:</h6>
          <table className="table table-bordered">
            <thead>
              <tr>
                <th>Retailer Name</th>
                <th>Email</th>
                <th>Phone</th>
                <th>PAN</th>
                <th>GST</th>
                <th>Aadhar</th>
              </tr>
            </thead>
            <tbody>
              {newlyCreated.map((distributor, index) => (
                <tr key={index}>
                  <td>{distributor?.Kyc?.name || "N/A"}</td>
                  <td>{distributor?.Kyc?.email || "N/A"}</td>
                  <td>{distributor?.Kyc?.mobile_number || "N/A"}</td>
                  <td>{distributor?.Kyc?.pan_number || "N/A"}</td>
                  <td>{distributor?.Kyc?.gst_number || "N/A"}</td>
                  <td>{distributor?.Kyc?.aadhar_number || "N/A"}</td>
                </tr>
              ))}
            </tbody>
          </table>
        </div>
      )}
    </div>
  );
};

export default AddMultiplerd;
